@@ -8,6 +8,7 @@ import torch.optim as optim
 import torch.nn as nn
 
 import numpy as np
+import math
 import pandas as pd
 import seaborn as sns
 import matplotlib
@@ -15,140 +16,39 @@ import matplotlib.pyplot as plt
 from .function_basis import RealSphericalHarmonics
 
 from .networks import FCNN
-from .version_utils import warn_deprecate_class
-from .generator import Generator3D, GeneratorSpherical
+from ._version_utils import warn_deprecate_class
+from .generators import Generator3D, GeneratorSpherical
+from .conditions import NoCondition, DirichletBVPSpherical, InfDirichletBVPSpherical
+from .conditions import DirichletBVPSphericalBasis, InfDirichletBVPSphericalBasis
 from inspect import signature
 from copy import deepcopy
 from datetime import datetime
 
-# the name ExampleGenerator3D is deprecated
+# generators defined in this module have been move to generators.py (and renamed)
 ExampleGenerator3D = warn_deprecate_class(Generator3D)
-
-# the name ExampleGeneratorSpherical is deprecated
 ExampleGeneratorSpherical = warn_deprecate_class(GeneratorSpherical)
 
+# conditions defined in this module have been moved to conditions.py (and renamed)
+NoConditionSpherical = warn_deprecate_class(NoCondition)
+NoConditionSphericalHarmonics = warn_deprecate_class(NoCondition)
+DirichletBVPSpherical = warn_deprecate_class(DirichletBVPSpherical)
+DirichletBVPSphericalHarmonics = warn_deprecate_class(DirichletBVPSphericalBasis)
+InfDirichletBVPSpherical = warn_deprecate_class(InfDirichletBVPSpherical)
+InfDirichletBVPSphericalHarmonics = warn_deprecate_class(InfDirichletBVPSphericalBasis)
+
 
 def _nn_output_spherical_input(net, rs, thetas, phis):
     points = torch.cat((rs, thetas, phis), 1)
     return net(points)
 
 
-class BaseConditionSpherical:
-    def enforce(self, net, r, theta, phi):
-        raise NotImplementedError(f"Abstract {self.__class__.__name__} cannot be enforced")
-
-
-class NoConditionSpherical(BaseConditionSpherical):
-    def __init__(self):
-        pass
-
-    def enforce(self, net, r, theta, phi):
-        return _nn_output_spherical_input(net, r, theta, phi)
-
-
-class DirichletBVPSpherical(BaseConditionSpherical):
-    """Dirichlet boundary condition for the interior and exterior boundary of the sphere, where the interior boundary is not necessarily a point
-        We are solving :math:`u(t)` given :math:`u(r, \\theta, \\phi)\\bigg|_{r = r_0} = f(\\theta, \\phi)` and :math:`u(r, \\theta, \\phi)\\bigg|_{r = r_1} = g(\\theta, \\phi)`
-
-    :param r_0: The radius of the interior boundary. When r_0 = 0, the interior boundary is collapsed to a single point (center of the ball)
-    :type r_0: float
-    :param f: The value of :math:u on the interior boundary. :math:`u(r, \\theta, \\phi)\\bigg|_{r = r_0} = f(\\theta, \\phi)`.
-    :type f: callable
-    :param r_1: The radius of the exterior boundary; if set to None, `g` must also be None
-    :type r_1: float or None
-    :param g: The value of :math:u on the exterior boundary. :math:`u(r, \\theta, \\phi)\\bigg|_{r = r_1} = g(\\theta, \\phi)`. If set to None, `r_1` must also be set to None
-    :type g: callable or None
-    """
-
-    def __init__(self, r_0, f, r_1=None, g=None):
-        """Initializer method
-        """
-        if (r_1 is None) ^ (g is None):
-            raise ValueError(f'r_1 and g must be both/neither set to None; got r_1={r_1}, g={g}')
-        self.r_0, self.r_1 = r_0, r_1
-        self.f, self.g = f, g
-
-    def enforce(self, net, r, theta, phi):
-        r"""Enforce the output of a neural network to satisfy the boundary condition.
-
-        :param net: The neural network that approximates the ODE.
-        :type net: `torch.nn.Module`
-        :param r: The radii of points where the neural network output is evaluated.
-        :type r: `torch.tensor`
-        :param theta: The latitudes of points where the neural network output is evaluated. `theta` ranges [0, pi]
-        :type theta: `torch.tensor`
-        :param phi: The longitudes of points where the neural network output is evaluated. `phi` ranges [0, 2*pi)
-        :type phi: `torch.tensor`
-        :return: The modified output which now satisfies the boundary condition.
-        :rtype: `torch.tensor`
-
-
-        .. note::
-            `enforce` is meant to be called by the function `solve_spherical` and `solve_spherical_system`.
-        """
-        u = _nn_output_spherical_input(net, r, theta, phi)
-        if self.r_1 is None:
-            return (1 - torch.exp(-r + self.r_0)) * u + self.f(theta, phi)
-        else:
-            r_tilde = (r - self.r_0) / (self.r_1 - self.r_0)
-            # noinspection PyTypeChecker
-            return self.f(theta, phi) * (1 - r_tilde) + \
-                   self.g(theta, phi) * r_tilde + \
-                   (1. - torch.exp((1 - r_tilde) * r_tilde)) * u
-
-
-class InfDirichletBVPSpherical(BaseConditionSpherical):
-    """Similar to `DirichletBVPSpherical`; only difference is we are considering :math:`g(\\theta, \\phi)` as :math:`r_1 \\to \\infty`, so `r_1` doesn't need to be specified
-        We are solving :math:`u(t)` given :math:`u(r, \\theta, \\phi)\\bigg|_{r = r_0} = f(\\theta, \\phi)` and :math:`\\lim_{r \\to \\infty} u(r, \\theta, \\phi) = g(\\theta, \\phi)`
-
-    :param r_0: The radius of the interior boundary. When r_0 = 0, the interior boundary is collapsed to a single point (center of the ball)
-    :type r_0: float
-    :param f: The value of :math:u on the interior boundary. :math:`u(r, \\theta, \\phi)\\bigg|_{r = r_0} = f(\\theta, \\phi)`.
-    :type f: callable
-    :param g: The value of :math:u on the exterior boundary. :math:`u(r, \\theta, \\phi)\\bigg|_{r = r_1} = g(\\theta, \\phi)`.
-    :type g: callable
-    :param order: The smallest :math:k that guarantees :math:`\\lim_{r \\to +\\infty} u(r, \\theta, \\phi) e^{-k r} = 0`, defaults to 1
-    :type order: int or float, optional
-    """
-
-    def __init__(self, r_0, f, g, order=1):
-        self.r_0 = r_0
-        self.f = f
-        self.g = g
-        self.order = order
-
-    def enforce(self, net, r, theta, phi):
-        r"""Enforce the output of a neural network to satisfy the boundary condition.
-
-        :param net: The neural network that approximates the PDE.
-        :type net: `torch.nn.Module`
-        :param r: The radii of points where the neural network output is evaluated.
-        :type r: `torch.tensor`
-        :param theta: The latitudes of points where the neural network output is evaluated. `theta` ranges [0, pi]
-        :type theta: `torch.tensor`
-        :param phi: The longitudes of points where the neural network output is evaluated. `phi` ranges [0, 2*pi)
-        :type phi: `torch.tensor`
-        :return: The modified output which now satisfies the boundary condition.
-        :rtype: `torch.tensor`
-
-
-        .. note::
-            `enforce` is meant to be called by the function `solve_spherical` and `solve_spherical_system`.
-        """
-        u = _nn_output_spherical_input(net, r, theta, phi)
-        dr = r - self.r_0
-        return self.f(theta, phi) * torch.exp(-self.order * dr) + \
-               self.g(theta, phi) * torch.tanh(dr) + \
-               torch.exp(-self.order * dr) * torch.tanh(dr) * u
-
-
 class SolutionSpherical:
     """A solution to a PDE (system) in spherical coordinates
 
     :param nets: The neural networks that approximate the PDE.
     :type nets: list[`torch.nn.Module`]
     :param conditions: The conditions of the PDE (system).
-    :type conditions: list[`neurodiffeq.pde_spherical.BaseConditionSpherical`]
+    :type conditions: list[`neurodiffeq.conditions.BaseCondition`]
     """
 
     def __init__(self, nets, conditions):
@@ -164,23 +64,23 @@ class SolutionSpherical:
         """Evaluate the solution at certain points.
 
         :param rs: The radii of points where the neural network output is evaluated.
-        :type rs: `torch.tensor`
+        :type rs: `torch.Tensor`
         :param thetas: The latitudes of points where the neural network output is evaluated. `theta` ranges [0, pi]
-        :type thetas: `torch.tensor`
+        :type thetas: `torch.Tensor`
         :param phis: The longitudes of points where the neural network output is evaluated. `phi` ranges [0, 2*pi)
-        :type phis: `torch.tensor`
-        :param as_type: Whether the returned value is a `torch.tensor` ('tf') or `numpy.array` ('np').
+        :type phis: `torch.Tensor`
+        :param as_type: Whether the returned value is a `torch.Tensor` ('tf') or `numpy.array` ('np').
         :type as_type: str
         :return: dependent variables are evaluated at given points.
-        :rtype: list[`torch.tensor` or `numpy.array` (when there is more than one dependent variables)
-            `torch.tensor` or `numpy.array` (when there is only one dependent variable)
+        :rtype: list[`torch.Tensor` or `numpy.array` (when there is more than one dependent variables)
+            `torch.Tensor` or `numpy.array` (when there is only one dependent variable)
         """
         if not isinstance(rs, torch.Tensor):
-            rs = torch.tensor(rs, dtype=torch.float32)
+            rs = torch.tensor(rs)
         if not isinstance(thetas, torch.Tensor):
-            thetas = torch.tensor(thetas, dtype=torch.float32)
+            thetas = torch.tensor(thetas)
         if not isinstance(phis, torch.Tensor):
-            phis = torch.tensor(phis, dtype=torch.float32)
+            phis = torch.tensor(phis)
         original_shape = rs.shape
         rs = rs.reshape(-1, 1)
         thetas = thetas.reshape(-1, 1)
@@ -210,7 +110,7 @@ def solve_spherical(
             then `pde` should be a function that maps :math:`(u, r, \\theta, \\phi)` to :math:`F(u, r,\\theta, \\phi)`
         :type pde: callable
         :param condition: The initial/boundary condition that :math:`u` should satisfy.
-        :type condition: `neurodiffeq.pde_spherical.BaseConditionSpherical`
+        :type condition: `neurodiffeq.conditions.BaseCondition`
         :param r_min: radius for inner boundary; ignored if both generators are provided; optional
         :type r_min: float
         :param r_max: radius for outer boundary; ignored if both generators are provided; optional
@@ -245,6 +145,10 @@ def solve_spherical(
             Optionally, MSE against analytic solution, the nets, conditions, training generator, validation generator, optimizer and loss function.
             The solution is a function that has the signature `solution(xs, ys, as_type)`.
         :rtype: tuple[`neurodiffeq.pde_spherical.SolutionSpherical`, dict]; or tuple[`neurodiffeq.pde_spherical.SolutionSpherical`, dict, dict]; or tuple[`neurodiffeq.pde_spherical.SolutionSpherical`, dict, dict, dict]
+
+
+        .. note::
+            This function is deprecated, use a `SphericalSolver` instead
         """
 
     warnings.warn("solve_spherical is deprecated, consider using SphericalSolver instead")
@@ -277,7 +181,7 @@ def solve_spherical_system(
             then `pde_system` should be a function that maps :math:`(u_1, u_2, ..., u_n, r, \\theta, \\phi)` to a list where the i-th entry is :math:`F_i(u_1, u_2, ..., u_n, r, \\theta, \\phi)`.
         :type pde_system: callable
         :param conditions: The initial/boundary conditions. The ith entry of the conditions is the condition that :math:`u_i` should satisfy.
-        :type conditions: list[`neurodiffeq.pde_spherical.BaseConditionSpherical`]
+        :type conditions: list[`neurodiffeq.conditions.BaseCondition`]
         :param r_min: radius for inner boundary; ignored if both generators are provided; optional
         :type r_min: float
         :param r_max: radius for outer boundary; ignored if both generators are provided; optional
@@ -312,9 +216,19 @@ def solve_spherical_system(
             Optionally, MSE against analytic solutions, the nets, conditions, training generator, validation generator, optimizer and loss function.
             The solution is a function that has the signature `solution(xs, ys, as_type)`.
         :rtype: tuple[`neurodiffeq.pde_spherical.SolutionSpherical`, dict]; or tuple[`neurodiffeq.pde_spherical.SolutionSpherical`, dict, dict]; or tuple[`neurodiffeq.pde_spherical.SolutionSpherical`, dict, dict, dict]
+
+        .. note::
+            This function is deprecated, use a `SphericalSolver` instead
         """
     warnings.warn("solve_spherical_system is deprecated, consider using SphericalSolver instead")
 
+    if harmonics_fn is None:
+        def enforcer(net, cond, points):
+            return cond.enforce(net, *points)
+    else:
+        def enforcer(net, cond, points):
+            return (cond.enforce(net, points[0]) * harmonics_fn(*points[1:])).sum(dim=1, keepdims=True)
+
     solver = SphericalSolver(
         pde_system=pde_system,
         conditions=conditions,
@@ -328,6 +242,7 @@ def solve_spherical_system(
         criterion=criterion,
         n_batches_train=1,
         n_batches_valid=1,
+        enforcer=enforcer,
         # deprecated arguments
         batch_size=batch_size,
         shuffle=shuffle,
@@ -351,7 +266,7 @@ class SphericalSolver:
     :param pde_system: the PDE system to solve; maps a tuple of three coordinates to a tuple of PDE residuals, both the coordinates and PDE residuals must have shape (-1, 1)
     :type pde_system: callable
     :param conditions: list of boundary conditions for each target function
-    :type conditions: list[`neurodiffeq.pde_spherical.BaseConditionSpherical`]
+    :type conditions: list[`neurodiffeq.conditions.BaseCondition`]
     :param r_min: radius for inner boundary; ignored if train_generator and valid_generator are both set; r_min > 0; optional
     :type r_min: float
     :param r_max: radius for outer boundary; ignored if train_generator and valid_generator are both set; r_max > r_min; optional
@@ -462,21 +377,22 @@ class SphericalSolver:
 
     @property
     def global_epoch(self):
-        """global epoch count, always equal to the length of train loss history
+        """Global epoch count, always equal to the length of train loss history
+
         :return: number of training epochs that have been run
         :rtype: int
         """
         return len(self.loss['train'])
 
     def _auto_enforce(self, net, cond, *points):
-        """enforce condition on network with inputs
-        if self.enforcer is set, use it;
-        otherwise, fill cond.enforce() with as many arguments as needed
+        """Enforce condition on network with inputs.
+            If self.enforcer is set, use it;
+            otherwise, fill cond.enforce() with as many arguments as needed
 
         :param net: network for parameterized solution
         :type net: torch.nn.Module
         :param cond: condition (a.k.a. parameterization) for the network
-        :type cond: `neurodiffeq.pde_spherical.BaseConditionSpherical`
+        :type cond: `neurodiffeq.conditions.BaseCondition`
         :param points: a tuple of vectors, each with shape = (-1, 1)
         :type points: tuple[torch.Tensor]
         :return: function values at sampled points
@@ -490,7 +406,7 @@ class SphericalSolver:
         return cond.enforce(net, *points)
 
     def _update_history(self, value, metric_type, key):
-        """append a value to corresponding history list
+        """Append a value to corresponding history list
 
         :param value: value to be appended
         :type value: float
@@ -508,15 +424,15 @@ class SphericalSolver:
             raise KeyError(f'history type = {metric_type} not understood')
 
     def _update_train_history(self, value, metric_type):
-        """append a value to corresponding training history list"""
+        """Append a value to corresponding training history list"""
         self._update_history(value, metric_type, key='train')
 
     def _update_valid_history(self, value, metric_type):
-        """append a value to corresponding validation history list"""
+        """Append a value to corresponding validation history list"""
         self._update_history(value, metric_type, key='valid')
 
     def _generate_batch(self, key):
-        """generate the next batch, register in self._batch_examples and return the batch
+        """Generate the next batch, register in self._batch_examples and return the batch
 
         :param key: {'train', 'valid'}; dict key in self._examples / self._batch_examples / self._batch_start
         :type key: str
@@ -528,16 +444,16 @@ class SphericalSolver:
         return self._batch_examples[key]
 
     def _generate_train_batch(self):
-        """generate the next training batch, register in self._batch_examples and return"""
+        """Generate the next training batch, register in self._batch_examples and return"""
         return self._generate_batch('train')
 
     def _generate_valid_batch(self):
-        """generate the next validation batch, register in self._batch_examples and return"""
+        """Generate the next validation batch, register in self._batch_examples and return"""
         return self._generate_batch('valid')
 
     def _do_optimizer_step(self):
-        """Optimization procedures after gradients have been computed. Usually, self.optimizer.step() is sufficient.
-        At times, user can overwrite this method to perform gradient clipping, etc. Here is an example:
+        r"""Optimization procedures after gradients have been computed. Usually, self.optimizer.step() is sufficient.
+            At times, user can overwrite this method to perform gradient clipping, etc. Here is an example:
         >>> import itertools
         >>> class MySolver(SphericalSolver)
         >>>     def _do_optimizer_step(self):
@@ -547,7 +463,7 @@ class SphericalSolver:
         self.optimizer.step()
 
     def _run_epoch(self, key):
-        """run an epoch on train/valid points, update history, and perform an optimization step if key=='train'
+        """Run an epoch on train/valid points, update history, and perform an optimization step if key=='train'
         Note that the optimization step is only performed after all batches are run
         This method doesn't resample points, which shall be handled in the `.fit()` call.
 
@@ -601,25 +517,26 @@ class SphericalSolver:
             self._update_history(epoch_analytic_mse, 'analytic_mse', key)
 
     def run_train_epoch(self):
-        """run a training epoch, update history, and perform gradient descent"""
+        """Run a training epoch, update history, and perform gradient descent"""
         self._run_epoch('train')
 
     def run_valid_epoch(self):
-        """run a validation epoch and update history"""
+        """Run a validation epoch and update history"""
         self._run_epoch('valid')
 
     def _update_best(self):
-        """update self.lowest_loss and self.best_nets if current validation loss is lower than self.lowest_loss"""
+        """Update self.lowest_loss and self.best_nets if current validation loss is lower than self.lowest_loss"""
         current_loss = self.loss['valid'][-1]
         if (self.lowest_loss is None) or current_loss < self.lowest_loss:
             self.lowest_loss = current_loss
             self.best_nets = deepcopy(self.nets)
 
     def fit(self, max_epochs, callbacks=None, monitor=None):
-        """Run multiple epochs of training and validation, update best loss at the end of each epoch.
-        This method does not return solution, which is done in the `.get_solution` method.
-        If `callbacks` is passed, callbacks are run one at a time, after training, validating, updaing best model and before monitor checking
-        A callback function `cb(solver)` can set `solver._stop_training` to True to perform early stopping,
+        r"""Run multiple epochs of training and validation, update best loss at the end of each epoch.
+            This method does not return solution, which is done in the `.get_solution` method.
+            If `callbacks` is passed, callbacks are run one at a time, after training, validating, updaing best model and before monitor checking
+            A callback function `cb(solver)` can set `solver._stop_training` to True to perform early stopping,
+
         :param max_epochs: number of epochs to run
         :type max_epochs: int
         :param monitor: DEPRECATED; use a MonitorCallback instance instead; Monitor for visualizing solution and metrics
@@ -657,7 +574,8 @@ class SphericalSolver:
                     )
 
     def get_solution(self, copy=True, best=True, harmonics_fn=None):
-        """return a solution class
+        """Return a solution class
+
         :param copy: if True, use a deep copy of internal nets and conditions
         :type copy: bool
         :param best: if True, return the solution with lowest loss instead of the solution after the last epoch
@@ -679,11 +597,11 @@ class SphericalSolver:
             return SolutionSpherical(nets, conditions)
 
     def get_internals(self, param_names, return_type='list'):
-        """return internal variable(s) of the solver
-        if param_names == 'all', return all internal variables as a dict;
-        if param_names is single str, return the corresponding variables
-        if param_names is a list and return_type == 'list', return corresponding internal variables as a list
-        if param_names is a list and return_type == 'dict', return a dict with keys in param_names
+        """Return internal variable(s) of the solver
+        If param_names == 'all', return all internal variables as a dict;
+        If param_names is single str, return the corresponding variables
+        If param_names is a list and return_type == 'list', return corresponding internal variables as a list
+        If param_names is a list and return_type == 'dict', return a dict with keys in param_names
 
         :param param_names: a parameter name or a list of parameter names
         :type param_names: str or list[str]
@@ -726,8 +644,9 @@ class SphericalSolver:
             raise ValueError(f"unrecognized return_type = {return_type}")
 
     def additional_loss(self, funcs, key):
-        """return additional loss; this method is to be overridden by subclasses
-        This method can use any of the internal variables: the current batch, the nets, the conditions, etc.
+        r"""Return additional loss; this method is to be overridden by subclasses
+            This method can use any of the internal variables: the current batch, the nets, the conditions, etc.
+
         :param funcs: outputs of the networks after enforced by conditions
         :type funcs: list[torch.Tensor]
         :param key: {'train', 'valid'}; phase of the epoch; used to access the sample batch, etc.
@@ -764,7 +683,7 @@ class MonitorSpherical:
     """
 
     def __init__(self, r_min, r_max, check_every=100, var_names=None, shape=(10, 10, 10), r_scale='linear',
-                 theta_min=0.0, theta_max=np.pi, phi_min=0.0, phi_max=np.pi * 2):
+                 theta_min=0.0, theta_max=math.pi, phi_min=0.0, phi_max=math.pi * 2):
         """Initializer method
         """
         self.contour_plot_available = self._matplotlib_version_satisfies()
@@ -814,7 +733,7 @@ class MonitorSpherical:
 
     @staticmethod
     def _longitude_formatter(value, count):
-        value = int(round(value / np.pi * 180)) - 180
+        value = int(round(value / math.pi * 180)) - 180
         if value == 0 or abs(value) == 180:
             marker = ''
         elif value > 0:
@@ -825,7 +744,7 @@ class MonitorSpherical:
 
     @staticmethod
     def _latitude_formatter(value, count):
-        value = int(round(value / np.pi * 180)) - 90
+        value = int(round(value / math.pi * 180)) - 90
         if value == 0:
             marker = ''
         elif value > 0:
@@ -852,7 +771,7 @@ class MonitorSpherical:
         :param nets: The neural networks that approximates the PDE.
         :type nets: list [`torch.nn.Module`]
         :param conditions: The initial/boundary condition of the PDE.
-        :type conditions: list [`neurodiffeq.pde_spherical.BaseConditionSpherical`]
+        :type conditions: list [`neurodiffeq.conditions.BaseCondition`]
         :param loss_history: The history of training loss and validation loss. The 'train' entry is a list of training loss and 'valid' entry is a list of validation loss.
         :type loss_history: dict['train': list[float], 'valid': list[float]]
         :param analytic_mse_history: The history of training and validation MSE against analytic solution. The 'train' entry is a list of training analytic MSE and 'valid' entry is a list of validation analytic MSE.
@@ -930,7 +849,7 @@ class MonitorSpherical:
             plt.pause(0.05)
 
     def customization(self):
-        """customized tweaks can be implemented by overwriting this method
+        """Customized tweaks can be implemented by overwriting this method.
         """
         pass
 
@@ -961,11 +880,11 @@ class MonitorSpherical:
             theta = self.theta_label.reshape(*self.shape)[0, :, 0]
             phi = self.phi_label.reshape(*self.shape)[0, 0, :]
             cax = ax.contourf(phi, theta, u, cmap='magma')
-            ax.xaxis.set_major_locator(plt.MultipleLocator(np.pi / 6))
-            ax.xaxis.set_minor_locator(plt.MultipleLocator(np.pi / 12))
+            ax.xaxis.set_major_locator(plt.MultipleLocator(math.pi / 6))
+            ax.xaxis.set_minor_locator(plt.MultipleLocator(math.pi / 12))
             ax.xaxis.set_major_formatter(plt.FuncFormatter(self._longitude_formatter))
-            ax.yaxis.set_major_locator(plt.MultipleLocator(np.pi / 6))
-            ax.yaxis.set_minor_locator(plt.MultipleLocator(np.pi / 12))
+            ax.yaxis.set_major_locator(plt.MultipleLocator(math.pi / 6))
+            ax.yaxis.set_minor_locator(plt.MultipleLocator(math.pi / 12))
             ax.yaxis.set_major_formatter(plt.FuncFormatter(self._latitude_formatter))
             ax.grid(which='major', linestyle='--', linewidth=0.5)
             ax.grid(which='minor', linestyle=':', linewidth=0.5)
@@ -997,8 +916,9 @@ class MonitorSpherical:
         return self
 
     def set_variable_count(self, n):
-        """manually set the number of scalar fields to be visualized;
-        if not set, defaults to length of `nets` passed to `self.check()` every time `self.check()` is called
+        r"""Manually set the number of scalar fields to be visualized;
+            If not set, defaults to length of `nets` passed to `self.check()` every time `self.check()` is called
+
         :param n: number of scalar fields to overwrite default
         :type n: int
         :return: self
@@ -1007,128 +927,15 @@ class MonitorSpherical:
         return self
 
     def unset_variable_count(self):
-        """manually unset the number of scalar fields to be visualized;
-        Once unset, the number defaults to length of `nets` passed to `self.check()` every time `self.check()` is called
+        r"""Manually unset the number of scalar fields to be visualized;
+            Once unset, the number defaults to length of `nets` passed to `self.check()` every time `self.check()` is called
+
         :return: self
         """
         self.n_vars = None
         return self
 
 
-class BaseConditionSphericalHarmonics(BaseConditionSpherical):
-    """
-    :param max_degree: highest degree for spherical harmonics
-    :type max_degree: int
-    """
-
-    def __init__(self, max_degree=4):
-        self.max_degree = max_degree
-
-    # noinspection PyMethodOverriding
-    def enforce(self, net, r):
-        raise NotImplementedError(f'Abstract BVP {self.__class__.__name__} cannot be enforced')
-
-
-class NoConditionSphericalHarmonics(BaseConditionSphericalHarmonics):
-    def enforce(self, net, r):
-        return net(r)
-
-
-class DirichletBVPSphericalHarmonics(BaseConditionSphericalHarmonics):
-    """Similar to `DirichletBVPSpherical`; only difference is this condition is enforced on a neural net that takes in :math:r and returns the spherical harmonic coefficients R(r)
-        i.e., we constrain the coefficients :math:`R(r)` of spherical harmonics instead of the inner product :math:`R(r) \\cdot Y(\\theta, \\phi)`
-        We are solving :math:`R(r)` given :math:`R(r)\\bigg|_{r = r_0} = R_0` and :math:`R(r)\\bigg|_{r = r_1} = R_1`.
-
-    :param r_0: The radius of the interior boundary. When r_0 = 0, the interior boundary is collapsed to a single point (center of the ball)
-    :type r_0: float
-    :param R_0: The value of harmonic coefficients :math:R on the interior boundary. :math:`R(r)\\bigg|_{r = r_0} = R_0`.
-    :type R_0: torch.tensor
-    :param r_1: The radius of the exterior boundary; if set to None, `R_1` must also be None
-    :type r_1: float or None
-    :param R_1: The value of harmonic coefficients :math:R on the exterior bounadry. :math:`R(r)\\bigg|_{r = r_1} = R_1`.
-    :type R_1: torch.tensor
-    :param max_degree: highest degree for spherical harmonics
-    :type max_degree: int
-    """
-
-    def __init__(self, r_0, R_0, r_1=None, R_1=None, max_degree=4):
-        """Initializer method
-        """
-        super(DirichletBVPSphericalHarmonics, self).__init__(max_degree=max_degree)
-        if (r_1 is None) ^ (R_1 is None):
-            raise ValueError(f'r_1 and R_1 must be both/neither set to None; got r_1={r_1}, R_1={R_1}')
-        self.r_0, self.r_1 = r_0, r_1
-        self.R_0, self.R_1 = R_0, R_1
-
-    def enforce(self, net, r):
-        r"""Enforce the output of a neural network to satisfy the boundary condition.
-
-        :param net: The neural network that approximates the coefficients for spherical harmonics.
-        :type net: `torch.nn.Module`
-        :param r: The radii of points where the neural network output is evaluated.
-        :type r: `torch.tensor`
-        :return: The modified output which now satisfies the boundary condition.
-        :rtype: `torch.tensor`
-
-
-        .. note::
-            `enforce` is meant to be called by the function `solve_spherical` and `solve_spherical_system`.
-        """
-        R_raw = net(r)
-        if self.r_1 is None:
-            # noinspection PyTypeChecker
-            ret = (1 - torch.exp(-r + self.r_0)) * R_raw + self.R_0
-        else:
-            r_tilde = (r - self.r_0) / (self.r_1 - self.r_0)
-            # noinspection PyTypeChecker
-            ret = self.R_0 * (1 - r_tilde) + self.R_1 * r_tilde + (1. - torch.exp((1 - r_tilde) * r_tilde)) * R_raw
-        return ret
-
-
-class InfDirichletBVPSphericalHarmonics(BaseConditionSphericalHarmonics):
-    """Similar to `InfDirichletBVPSpherical`; only difference is this condition is enforced on a neural net that takes in :math:r and returns the spherical harmonic coefficients R(r)
-        i.e., we constrain the coefficients :math:`R(r)` of spherical harmonics instead of the inner product :math:`R(r) \\cdot Y(\\theta, \\phi)`
-        We are solving :math:`R(r)` given :math:`R(r)\\bigg|_{r = r_0} = R_0` and :math:`\\lim_{r \\to \\infty} R(r) = R_\\infty`
-
-    :param r_0: The radius of the interior boundary. When r_0 = 0, the interior boundary is collapsed to a single point (center of the ball)
-    :type r_0: float
-    :param R_0: The value of harmonic coefficients :math:R on the interior boundary. :math:`R(r)\\bigg|_{r = r_0} = R_0`.
-    :type R_0: torch.tensor
-    :param R_inf: The value of harmonic coefficients :math:R at infinity. :math:`\\lim_{r \\to \\infty} R(r) = R_\\infty`.
-    :type R_inf: torch.tensor
-    :param order: The smallest :math:k that guarantees :math:`\\lim_{r \\to +\\infty} R(r) e^{-k r} = \\bf 0`, defaults to 1
-    :type order: int or float, optional
-    :param max_degree: highest degree for spherical harmonics
-    :type max_degree: int
-    """
-
-    def __init__(self, r_0, R_0, R_inf, order=1, max_degree=4):
-        super(InfDirichletBVPSphericalHarmonics, self).__init__(max_degree=max_degree)
-        self.r_0 = r_0
-        self.R_0 = R_0
-        self.R_inf = R_inf
-        self.order = order
-
-    def enforce(self, net, r):
-        r"""Enforce the output of a neural network to satisfy the boundary condition.
-
-        :param net: The neural network that approximates the coefficients for the spherical harmonics.
-        :type net: `torch.nn.Module`
-        :param r: The radii of points where the neural network output is evaluated.
-        :type r: `torch.tensor`
-        :return: The modified output which now satisfies the boundary condition.
-        :rtype: `torch.tensor`
-
-        .. note::
-            `enforce` is meant to be called by the function `solve_spherical` and `solve_spherical_system`.
-        """
-        R_raw = net(r)
-        dr = r - self.r_0
-        return self.R_0 * torch.exp(-self.order * dr) + \
-               self.R_inf * torch.tanh(dr) + \
-               torch.exp(-self.order * dr) * torch.tanh(dr) * R_raw
-
-
 class SolutionSphericalHarmonics(SolutionSpherical):
     """
     A solution to a PDE (system) in spherical coordinates
@@ -1136,7 +943,7 @@ class SolutionSphericalHarmonics(SolutionSpherical):
     :param nets: list of networks that takes in radius tensor and outputs the coefficients of spherical harmonics
     :type nets: list[`torch.nn.Module`]
     :param conditions: list of conditions to be enforced on each nets; must be of the same length as nets
-    :type conditions: list[BaseConditionSphericalHarmonics]
+    :type conditions: list[`neurodiffeq.conditions.BaseCondition`]
     :param harmonics_fn: mapping from :math:`\\theta` and :math:`\\phi` to basis functions, e.g., spherical harmonics
     :type harmonics_fn: callable
     :param max_degree: DEPRECATED and SUPERSEDED by harmonics_fn; highest used for the harmonic basis
@@ -1190,7 +997,7 @@ class MonitorSphericalHarmonics(MonitorSpherical):
     """
 
     def __init__(self, r_min, r_max, check_every=100, var_names=None, shape=(10, 10, 10), r_scale='linear',
-                 harmonics_fn=None, theta_min=0.0, theta_max=np.pi, phi_min=0.0, phi_max=np.pi * 2,
+                 harmonics_fn=None, theta_min=0.0, theta_max=math.pi, phi_min=0.0, phi_max=math.pi * 2,
                  # DEPRECATED
                  max_degree=None):
         super(MonitorSphericalHarmonics, self).__init__(
@@ -1240,6 +1047,7 @@ class MonitorSphericalHarmonics(MonitorSpherical):
 
 class MonitorCallback:
     """A callback for updating the monitor plots (and optionally saving the fig to disk)
+
     :param monitor: the underlying monitor responsible for plotting solutions
     :type monitor: MonitorSpherical
     :param fig_dir: directory for saving monitor figs; if not specified, figs will not be saved
